==> laboneq-2.40.0-cp39-abi3-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: laboneq/compiler/seqc/awg_sampled_event.py <==
# Copyright 2023 Zurich Instruments AG
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum, auto
from typing import Any, Callable

from sortedcollections import SortedDict


class AWGEventType(Enum):
    LOOP_STEP_START = auto()
    LOOP_STEP_END = auto()
    PUSH_LOOP = auto()
    ITERATE = auto()
    SEQUENCER_START = auto()
    RESET_PRECOMPENSATION_FILTERS = auto()
    RESET_PRECOMPENSATION_FILTERS_END = auto()
    INITIAL_RESET_PHASE = auto()
    RESET_PHASE = auto()
    SET_OSCILLATOR_FREQUENCY = auto()
    ACQUIRE = auto()
    QA_EVENT = auto()
    TRIGGER_OUTPUT = auto()
    SWITCH_OSCILLATOR = auto()
    MATCH = auto()
    PLAY_WAVE = auto()
    PLAY_HOLD = auto()
    INIT_AMPLITUDE_REGISTER = auto()
    CHANGE_OSCILLATOR_PHASE = auto()
    SETUP_PRNG = auto()
    DROP_PRNG_SETUP = auto()
    PRNG_SAMPLE = auto()
    DROP_PRNG_SAMPLE = auto()
    PPC_SWEEP_STEP_START = auto()
    PPC_SWEEP_STEP_END = auto()


@dataclass
class AWGEvent:
    type: AWGEventType
    start: int | None = None
    end: int | None = None
    priority: int | None = None
    params: dict[str, Any] = field(default_factory=dict)

    def frozen(self) -> frozenset:
        return frozenset(
            (self.type, self.start, self.end, frozenset(self.params.items()))
        )


@dataclass
class AWGSampledEventSequence:
    """Ordered mapping of the AWG timestamp in device samples to the events at that sample."""

    sequence: dict[int, list[AWGEvent]] = field(default_factory=SortedDict)

    def add(self, ts: int, event: AWGEvent):
        events_at_ts = self.sequence.setdefault(ts, [])
        events_at_ts.append(event)

    def merge(self, other: AWGSampledEventSequence):
        for ts, other_events in other.sequence.items():
            events_at_ts = self.sequence.setdefault(ts, [])
            events_at_ts.extend(other_events)

    def has_matching_event(self, predicate: Callable[[AWGEvent], bool]) -> bool:
        return next(
            (
                True
                for sampled_event_list in self.sequence.values()
                for x in sampled_event_list
                if predicate(x)
            ),
            False,
        )

==> laboneq-2.52.0-cp39-abi3-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: laboneq/compiler/seqc/awg_sampled_event.py <==
# Copyright 2023 Zurich Instruments AG
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum, auto
from typing import Any, Callable

from laboneq.compiler.seqc.signatures import PlaybackSignature


class AWGEventType(Enum):
    LOOP_STEP_START = auto()
    LOOP_STEP_END = auto()
    PUSH_LOOP = auto()
    ITERATE = auto()
    RESET_PRECOMPENSATION_FILTERS = auto()
    RESET_PRECOMPENSATION_FILTERS_END = auto()
    INITIAL_RESET_PHASE = auto()
    RESET_PHASE = auto()
    SET_OSCILLATOR_FREQUENCY = auto()
    ACQUIRE = auto()
    QA_EVENT = auto()
    TRIGGER_OUTPUT = auto()
    MATCH = auto()
    PLAY_WAVE = auto()
    PLAY_HOLD = auto()
    INIT_AMPLITUDE_REGISTER = auto()
    CHANGE_OSCILLATOR_PHASE = auto()
    SETUP_PRNG = auto()
    DROP_PRNG_SETUP = auto()
    PRNG_SAMPLE = auto()
    DROP_PRNG_SAMPLE = auto()
    PPC_SWEEP_STEP_START = auto()
    PPC_SWEEP_STEP_END = auto()


@dataclass
class AWGEvent:
    type: AWGEventType
    start: int | None = None
    end: int | None = None
    priority: int | None = None
    params: dict[str, Any] = field(default_factory=dict)

    def frozen(self) -> frozenset:
        return frozenset(
            (self.type, self.start, self.end, frozenset(self.params.items()))
        )

    @property
    def signature(self) -> PlaybackSignature:
        return self.params["playback_signature"]

    @property
    def maybe_signature(self) -> PlaybackSignature | None:
        return self.params.get("playback_signature")


@dataclass
class AWGSampledEventSequence:
    """Mapping of the AWG timestamp in device samples to the events at that sample.

    Use `.sort()` whenever sorted order is required.
    """

    sequence: dict[int, list[AWGEvent]] = field(default_factory=dict)

    def add(self, ts: int, event: AWGEvent):
        if ts in self.sequence:
            self.sequence[ts].append(event)
        else:
            self.sequence[ts] = [event]

    def merge(self, other: AWGSampledEventSequence):
        for ts, other_events in other.sequence.items():
            if ts in self.sequence:
                self.sequence[ts].extend(other_events)
            else:
                self.sequence[ts] = other_events

    def has_matching_event(self, predicate: Callable[[AWGEvent], bool]) -> bool:
        return next(
            (
                True
                for sampled_event_list in self.sequence.values()
                for x in sampled_event_list
                if predicate(x)
            ),
            False,
        )

    def sort(self):
        self.sequence = {ts: self.sequence[ts] for ts in sorted(self.sequence)}
